﻿using System;
using System.Collections.Generic;
using TeamPulseDataServiceSample.TeamPulse;
using System.Net;
using System.Data.Services.Client;

namespace TeamPulseDataServiceSample
{
    class Program
    {
        static TitanDataContext dataContext;
        static bool gotStoryAcceptanceCritierias;
        static bool gotProblems;
        static List<Problem> problems = new List<Problem>();

        static void Main(string[] args)
        {
            // CHANGE THIS URL TO A SERVICE THAT YOU'RE TESTING AGAINST AND THEN UPDATE THE SERVICE REFERENCE
            var serviceUrl = "http://localhost:9898/Services/DataService.svc";

            dataContext = new TitanDataContext(new Uri(serviceUrl, UriKind.RelativeOrAbsolute));
            dataContext.Credentials = GetCredentials();
   
            GetStoryAcceptanceCriterias(1);            
            GetProblemAndProblemAcceptanceCriteria(1);

            while (!gotProblems && !gotStoryAcceptanceCritierias)
            {
                System.Threading.Thread.Sleep(1000);
            }

            UpdateAProblem();
            
            Console.WriteLine("All done.  Press any key to exit.");
            Console.ReadKey();
        }

        private static NetworkCredential GetCredentials()
        {
            // USING CURRENT USER'S CREDENTIALS
            return System.Net.CredentialCache.DefaultNetworkCredentials;
        }

        /// <summary>
        /// Example of a query to the TeamPulse data service.
        /// </summary>
        /// <remarks>
        /// Queries story acceptance criteria where the project id is X and  
        /// expand to eager load related story.  Only get the first 5 records.
        /// </remarks>
        private static void GetStoryAcceptanceCriterias(int projectID)
        {

            string uri = string.Format("StoryAcceptanceCriterias?$filter=ProjectID eq {0}&$expand=Story&$top=5", projectID);
            dataContext.BeginExecute<StoryAcceptanceCriteria>(new Uri(uri, UriKind.Relative), ar =>
            {
                foreach (StoryAcceptanceCriteria storyAcceptanceCriteria in dataContext.EndExecute<StoryAcceptanceCriteria>(ar))
                {
                    Console.WriteLine("Story acceptance criteria " + storyAcceptanceCriteria.Description + " retrieved with story " +
                        storyAcceptanceCriteria.Story.Name);
                }
                gotStoryAcceptanceCritierias = true;
            },
            null);
        }

        /// <summary>
        /// Example of a batch query to the TeamPulse data service.
        /// </summary>
        /// <remarks>
        /// Queries problem with filter.
        /// Within the same query also gets problem acceptance criteria based on project id.
        /// </remarks>
        private static void GetProblemAndProblemAcceptanceCriteria(int projectID)
        {
            var problemsRequest = new DataServiceRequest<Problem>(new Uri(string.Format("Problems?$filter=ProjectID eq {0}", projectID), UriKind.Relative));
            var acceptanceCriteriaRequest = new DataServiceRequest<ProblemAcceptanceCriteria>(new Uri(string.Format("ProblemAcceptanceCriterias?$filter=ProjectID eq {0}", projectID), UriKind.Relative));
            dataContext.BeginExecuteBatch(ar =>
            {
                var batchOperationResponse = dataContext.EndExecuteBatch(ar);

                foreach (QueryOperationResponse qoResponse in batchOperationResponse)
                {
                    if (qoResponse.Error != null)
                        throw qoResponse.Error;

                    if (qoResponse is QueryOperationResponse<Problem>)
                    {
                        foreach (Problem problem in ((QueryOperationResponse<Problem>)qoResponse))
                        {
                            problems.Add(problem);
                            Console.WriteLine("Retrieved problem " + problem.Name);
                        }
                    }
                    else if (qoResponse is QueryOperationResponse<ProblemAcceptanceCriteria>)
                    {
                        foreach (ProblemAcceptanceCriteria problemAcceptanceCriteria in ((QueryOperationResponse<ProblemAcceptanceCriteria>)qoResponse))
                        {
                            Console.WriteLine("Retrieved problem acceptance criteria" + problemAcceptanceCriteria.DescriptionPlainText);
                        }
                    }
                }

                gotProblems = true;
            },
        gotProblems,
        problemsRequest,
        acceptanceCriteriaRequest);
        }


        /// <summary>
        /// Example of a updating an entity and saving the changes back.
        /// </summary>
        /// <remarks>
        /// Make a change to the entity, mark it as dirty with the UpdateObject method and then save the changes back.
        /// </remarks>
        private static void UpdateAProblem()
        {
            var problemToUpdate = problems[0];
            problemToUpdate.Estimate = problemToUpdate.Estimate + 10;

            Console.WriteLine(string.Format("Updated problem {0} with an estimate of {1}", problemToUpdate.ProblemID, problemToUpdate.Estimate));
            
            // Mark object as dirty
            dataContext.UpdateObject(problemToUpdate);

            dataContext.SaveChanges(); // This can also be an async call using BeginSaveChanges

            Console.WriteLine("Changes saved");
        }
    }
}
